#ifndef asset_h
#define asset_h

#include "memory.h"

struct BM_Font;
struct Bitmap;
struct Fixed_Font;

#define assets_xmacro() \
	x( \
		asset_id_main_font, \
		asset_type_bitmap, \
		"font.img" \
	) \
	x( \
		asset_id_smol_font, \
		asset_type_bitmap, \
		"smolfont.img" \
	) \
	x( \
		asset_id_char, \
		asset_type_bitmap, \
		"char.img" \
	) \
	x( \
		asset_id_map, \
		asset_type_bitmap, \
		"map.img" \
	) \
	x( \
		asset_id_fx, \
		asset_type_bitmap, \
		"fx.img" \
	) \
	x( \
		asset_id_ui, \
		asset_type_bitmap, \
		"ui.img" \
	) \
	x( \
		asset_id_prop, \
		asset_type_bitmap, \
		"prop.img" \
	) \
	x( \
		asset_id_arms, \
		asset_type_bitmap, \
		"arms.img" \
	) \
	x( \
		asset_id_face, \
		asset_type_bitmap, \
		"face.img" \
	) \
	x( \
		asset_id_boss, \
		asset_type_bitmap, \
		"boss.img" \
	) \
	x( \
		asset_id_colmask, \
		asset_type_bitmap, \
		"mask.img" \
	) \

typedef enum {
#define x(id, type, path) id,
	assets_xmacro()
#undef x
	asset_id_count
} Asset_ID;

typedef enum {
	asset_type_bitmap = 0,
	asset_type_txt,
	asset_type_sound
} Asset_Type;

typedef struct {
	Asset_ID id;
	Asset_Type type;
	const void* payload;
	int payload_size;
} Asset;

typedef struct {
	Asset_Type type;
	const char* name;
} Asset_Meta;

typedef struct {
	const char* text;
	int len;
} Text;

typedef struct {
	int size;
	int offset;
	char* buffer;
} Asset_Data;

void preload_assets();
const Asset* get_asset(Asset_ID id);
const Asset_Meta* get_asset_meta(Asset_ID id);

const struct Bitmap* get_bitmap(Asset_ID id);
const        Text*   get_text  (Asset_ID id);

const struct BM_Font* get_main_font();
const struct Fixed_Font* get_smol_font();

Asset_Data* read_asset_data(
	const char* name,
	Arena* arena
);

#endif
