#ifndef components_h
#define components_h

#include "animation.h"
#include "asset.h"
#include "config.h"
#include "entity.h"
#include "item.h"
#include "rect.h"
#include "sprite.h"

typedef struct {
	int x, y;
} CPosition;

typedef struct {
	int x, y;
} CVelocity;

typedef struct CSprite {
	Asset_ID asset;
	Rectangle rect;
	int z;
} CSprite;

typedef struct {
	Animation_ID id;
	int frame;
	int timer;
} CAnimated;

typedef struct {
	int x, y, w, h;
} CCollider;

typedef struct {
	int grounded;
} CBody;

typedef enum {
	face_left = 0,
	face_right,
	face_up,
	face_down
} CFace;

typedef struct {
	int life;
} CLifetime;

typedef struct {
	Entity parent;
	int x, y;
} CFollow;

typedef struct {
	int hp;
	int hurting;
} CDamageable;

typedef struct {
	/* Least significant bit says if it's drag or ground
	 * friction. */
	int amt;
} CFriction;

typedef struct {
	int timer;
	int state;
} CMonster;

typedef struct {
	Sprite_ID sprite;
} CCorpse;

typedef struct {
	Entity next;
} CList;

typedef struct {
	char level[32];
} CConttrig;

typedef struct {
	Item item;
} CPickup;

typedef struct {
	char handle[cutscene_handle_size];
} CCutscene;

typedef struct {
	char cut[32];
} CCuttrig;

typedef struct {
	int type;
} CBoss;

typedef struct {
	int anim;
} CImpact;

enum {
	ctype_position   = 1 << 0,
	ctype_velocity   = 1 << 1,
	ctype_sprite     = 1 << 2,
	ctype_animated   = 1 << 3,
	ctype_collider   = 1 << 4,
	ctype_face       = 1 << 5,
	ctype_body       = 1 << 6,
	ctype_trigger    = 1 << 7,
	ctype_lifetime   = 1 << 8,
	ctype_pickup     = 1 << 9,
	ctype_damageable = 1 << 10,
	ctype_follow     = 1 << 11,
	ctype_monster    = 1 << 12,
	ctype_ghast      = 1 << 13,
	ctype_friction   = 1 << 14,
	ctype_blood      = 1 << 15,
	ctype_corpse     = 1 << 16,
	ctype_list       = 1 << 18,
	ctype_cop        = 1 << 19,
	ctype_projectile = 1 << 20,
	ctype_conttrig   = 1 << 21,
	ctype_cutscene   = 1 << 22,
	ctype_cuttrig    = 1 << 23,
	ctype_boss       = 1 << 24,
	ctype_impact     = 1 << 25
};

#endif
