#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "gen.h"
#include "map.h"

#define max_biomes 32

static const char* tile_strings[] = {
#define x(n, ox, oy) #n,
	map_tiles_xmacro
#undef x
};

static Map_Tile map_tile_from_string(const char* str, int l) {
	int i;
	for (i = 0; i < map_tiles_count; i++) {
		if ((int)strlen(tile_strings[i]) != l) continue;
		if (!strncmp(str, tile_strings[i], l)) return i;
	}
	fprintf(stderr, "Warning: Unknown tile '%s'.\n", str);
	return map_tile_air;
}

static Room_Template_Tile temp_tile_from_char(char c) {
	switch (c) {
		case ' ': return rt_tile_air;
		case '_': return rt_tile_any;
		case '#': return rt_tile_wall;
		case '+': return rt_tile_door;
		case '-': return rt_tile_platform;
		case 'w': return rt_tile_water;
		default:
			fprintf(stderr, "Warning: Unkown symbol '%c'.\n", c);
			return rt_tile_air;
	}
}

int main(int argc, char** argv) {
	int len, l, rc, x, y, row, col;
	char line[1024];
	const char* c, * s;
	char* e;
	FILE* infile, * outfile;
	Biome_Rule rule;
	Map_Tile* tile;

	if (argc < 3) {
		fprintf(stderr, "Usage: %s infile outfile.\n", argv[0]);
		return 1;
	}

	infile = fopen(argv[1], "r");
	if (!infile) {
		fprintf(stderr, "Failed to open %s.\n", argv[1]);
		return 2;
	}

	outfile = fopen(argv[2], "wb");
	if (!outfile) {
		fprintf(stderr, "Failed to open %s.\n", argv[2]);
		return 3;
	}

	rc = 0;
	fwrite(&rc, sizeof rc, 1, outfile);
	while (fgets(line, sizeof line, infile)) {
		len = strlen(line);
		while (line[len - 1] == '\n') { len--; }
		if (!memcmp(line, "#rule", 5)) {
			if (rc)
				fwrite(&rule, sizeof rule, 1, outfile);
			rule.result_count = 0;
			rule.chance = 0;
			rule.w = 0;
			rule.h = 0;
			rule.layer = map_layer_midground;
			y = gen_max_rule * gen_max_rule;
			for (x = 0; x < y; x++)
				rule.grid[x] = rt_tile_any;
			rc++;
		} else if (!memcmp(line, "results:", 8)) {
			c = line + 8;
			for (;;) {
				while (isspace(*c)) { c++; }
				s = c;
				l = 0;
				while (isalnum(*c) || *c == '_') { c++; l++; }
				if (rule.result_count >= gen_max_rule_results) {
					fprintf(stderr, "Too many results.\n");
					return 5;
				}
				tile = &rule.results[rule.result_count++];
				*tile = map_tile_from_string(s, l);
				if (*c == '\n') break;
			}
		} else if (!memcmp(line, "chance:", 7))
			rule.chance = strtol(line + 7, 0, 10);
		else if (!memcmp(line, "ref:", 4)) {
			rule.rx = (int)strtol(line + 4, &e, 10);
			rule.ry = (int)strtol(e, 0, 10);
		} else if (!memcmp(line, "layer:", 6)) {
			c = line + 6;
			while (isspace(*c)) c++;
			if (!memcmp(c, "foreground", 10))
				rule.layer = map_layer_foreground;
			else if (!memcmp(c, "midground", 9))
				rule.layer = map_layer_midground;
			else if (!memcmp(c, "background", 10))
				rule.layer = map_layer_background;
			else
				fprintf(stderr, "Warning: Unkown map layer %s.\n",c);
		} else if (len > 0) {
			row = rule.h++;
			for (col = 0; col < len; col++) {
				rule.grid[col + row * gen_max_rule] =
					temp_tile_from_char(line[col]);
			}
			if (col > rule.w) rule.w = col;
		}
	}
	if (rc)
		fwrite(&rule, sizeof rule, 1, outfile);
	fseek(outfile, 0, SEEK_SET);
	fwrite(&rc, 4, 1, outfile);

	fclose(outfile);
	fclose(infile);
	return 0;
}
