#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "render.h"

typedef struct {
	Colour pixel;
	unsigned short offset, size;
} Run;

int main(int argc, char** argv) {
	FILE* outfile, * infile;
	char bmp_magic[2];
	int ret = 0;
	unsigned bmp_offset;
	int bmp_w, bmp_h, x, y, i, j, s, rc, cs;
	unsigned short bmp_bits;
	Colour pixel, * buffer, * cbuf;
	unsigned char t;
	Run run;
	Run* runs;

	if (argc < 3) {
		fprintf(stderr, "Usage: %s infile outfile.\n", argv[0]);
		return 1;
	}

	infile = fopen(argv[1], "rb");
	if (!infile) {
		fprintf(stderr, "Failed to open %s.\n", argv[1]);
		return 2;
	}

	outfile = fopen(argv[2], "wb");
	if (!outfile) {
		fprintf(stderr, "Failed to open %s.\n", argv[2]);
		fclose(infile);
		return 2;
	}

	fread(bmp_magic, 2, 1, infile);
	if (bmp_magic[0] != 'B' || bmp_magic[1] != 'M') {
		ret = 3;
		fprintf(stderr, "Not a valid bitmap file.\n");
		goto end;
	}
	
	fseek(infile, 10, SEEK_SET);
	fread(&bmp_offset, 4, 1, infile);

	fseek(infile, 18, SEEK_SET);
	fread(&bmp_w,    4, 1, infile);
	fread(&bmp_h,    4, 1, infile);
	fseek(infile, 28, SEEK_SET);
	fread(&bmp_bits, 2, 1, infile);

	if (bmp_bits != 32) {
		ret = 4;
		fprintf(stderr, "Bitmap must have 32 bit pixels. %d\n", bmp_bits);
		goto end;
	}

	fseek(infile, bmp_offset, SEEK_SET);

	fwrite(&bmp_w, 1, 2, outfile);
	fwrite(&bmp_h, 1, 2, outfile);

	s = bmp_w * bmp_h;
	buffer = malloc(s * 4);
	runs = malloc(s * 8);
	cbuf = malloc(s * 4);
	/* Flip & read. */
	for (y = 0; y < bmp_h; y++) {
		for (x = 0; x < bmp_w; x++) {
			fread(&pixel, 1, 4, infile);
			t = pixel.r;
			pixel.r = pixel.b;
			pixel.b = t;
			if (pixel.a == 0) {
				pixel.r = 0;
				pixel.g = 0;
				pixel.b = 0;
			}
			buffer[x + (bmp_h - y - 1) * bmp_w] = pixel;
		}
	}
	/* Run-length encode. */
	rc = 0;
	cs = 0;
	for (i = 0; i < s; i++) {
		pixel = buffer[i];
		if (i + 1 < s && memcmp(&buffer[i + 1], &pixel, 4) == 0) {
			run.pixel = pixel;
			run.offset = i;
			run.size = 1;
			for (
				j = i + 1;
				j < s && memcmp(&buffer[j], &pixel, 4) == 0;
				j++, run.size++
			);
			i += run.size - 1;
			runs[rc++] = run;
		} else {
			cbuf[cs++] = pixel;
		}
	}
	fseek(outfile, 4, SEEK_SET);
	fwrite(&rc, 4, 1, outfile);
	fwrite(runs, 8, rc, outfile);
	fwrite(cbuf, 4, cs, outfile);
	free(buffer);
	free(runs);
	free(cbuf);

end:
	fclose(infile);
	fclose(outfile);
	return ret;
}
