#include <stdio.h>
#include "config.h"

#define magic_num 0x46464952

typedef struct {
	unsigned hdr;
	unsigned fsize;
	unsigned fmtid;
	unsigned fmtblockid;
	unsigned blocksize;
	unsigned short audfmt;
	unsigned short chancnt;
	unsigned freq;
	unsigned bps;
	unsigned short bpb;
	unsigned short bips;
	unsigned dblockid;
	unsigned dsize;
} WAV_Hdr;

int main(int argc, char** argv) {
	char buffer[32];
	int buffer_size, size, rs, i;
	FILE* infile, * outfile;
	WAV_Hdr hdr;

	buffer_size = sizeof buffer;

	if (argc < 3) {
		fprintf(stderr, "Usage: %s infile outfile.\n", argv[0]);
		return 1;
	}

	infile = fopen(argv[1], "rb");
	if (!infile) {
		fprintf(stderr, "Failed to open %s.\n", argv[1]);
		return 2;
	}

	fseek(infile, 0, SEEK_END);
	size = ftell(infile);
	rewind(infile);
	size -= fread(&hdr, 1, sizeof hdr, infile);
	if (hdr.hdr != magic_num) {
		fprintf(stderr, "Invalid WAV file.\n");
		fclose(infile);
		return 4;
	}
	if (hdr.freq != audio_sample_rate) {
		fprintf(
			stderr,
			"Only %d rate WAV files are supported.\n",
			audio_sample_rate
		);
		fclose(infile);
		return 5;
	}
	if (hdr.chancnt != 1) {
		fprintf(
			stderr,
			"Only single channel WAV files are supported.\n"
		);
		fclose(infile);
		return 6;
	}

	outfile = fopen(argv[2], "wb");
	if (!outfile) {
		fprintf(stderr, "Failed to open %s.\n", argv[2]);
		fclose(infile);
		return 7;
	}

	for (i = 0; i < size; i += rs) {
		rs = fread(buffer, 1, buffer_size, infile);
		fwrite(buffer, 1, rs, outfile);
	}

	fclose(outfile);
	fclose(infile);
	return 0;
}
