#include "asset.h"
#include "gamestate.h"
#include "menu.h"
#include "plat.h"
#include "rcache.h"
#include "render.h"
#include "sound.h"
#include "world.h"

extern void render_border(void);

void gamestate_credit_init(
	Gamestate* gamestate,
	World* world
) {
	gamestate->xscroll = 0;
	gamestate->yscroll = 0;
	init_map(&world->map, room_width, room_height);
	cache_map(&world->map);
	set_song("menu.pcm");
}

void gamestate_credit_update(
	Gamestate* gamestate,
	World* world
) {
	(void)world;
	gamestate->xscroll++;
	gamestate->yscroll++;
}

static int credit_scroll(Gamestate* g, int y) {
	return y - ((g->yscroll << 7) >> fbits);
}

static void credit_text(
	Gamestate* gamestate,
	const char* txt,
	int y
) {
	int x;
	Rectangle r;
	const BM_Font* f;
	f = get_main_font();
	y = credit_scroll(gamestate, y);
	r = text_rect(f, txt);
	x = 116 + (renderer_w - 116) / 2 - r.w / 2;
	rc_add_cmd_rfont_text(
		f,
		x,
		y,
		txt
	);
}

static void credit_text_right(
	Gamestate* gamestate,
	const char* txt,
	int y
) {
	int x;
	Rectangle r;
	const BM_Font* f;
	const Colour blue = { 200, 200, 255, 255 };
	f = get_main_font();
	y = credit_scroll(gamestate, y);
	r = text_rect(f, txt);
	x = 116 + (renderer_w - 116) / 2 - r.w / 2;
	rc_add_cmd_rfont_text_col(
		f,
		x,
		y,
		txt,
		blue
	);
}

void gamestate_credit_render(
	Gamestate* gamestate,
	World* world
) {
	const Bitmap* bm;
	const Bitmap* b_char;
	const Sprite* s_cloud;
	const Animation* a_char;
	const int pw = 100;
	int cs, cy, i, h, h3;
	Rectangle clip = { 16, 16, 0, renderer_h - 16 };
	Rectangle cr;
	Colour col = { 0xff, 0xff, 0xff, 0xff };
	clip.w = clip.x + pw;
	bm = get_bitmap(asset_id_ui);
	a_char = get_animation(animation_demonbird_fly_right);
	b_char = get_bitmap(a_char->bitmap);
	s_cloud = get_sprite(sprite_cloud);
	rc_add_cmd_clip(&clip);
	render_map_background(&world->map);
	cy = 100;
	for (i = 0; i < 3; i++) {
		cs = 16 +
			-((gamestate->xscroll * (i + 1)) %
			s_cloud->rect.w);
		rc_add_cmd_bmp_col(
			bm,
			cs,
			cy,
			&s_cloud->rect,
			col
		);
		rc_add_cmd_bmp_col(
			bm,
			cs + s_cloud->rect.w,
			cy,
			&s_cloud->rect,
			col
		);
		cr.x = 16;
		cr.y = cy + 13;
		cr.w = pw;
		cr.h = renderer_h;
		rc_add_cmd_rect(&cr, col);
		cy += 30;
		col.r -= 20;
		col.g -= 20;
		col.b -= 20;
	}
	rc_add_cmd_bmp(
		b_char,
		66,
		100 + ((fsin(gamestate->yscroll * 3) * 20) >> fbits),
		&a_char->frames[
			gamestate->yscroll % a_char->frame_count
		]
	);
	rc_add_cmd_reset_clip();
	h  = get_main_font()->height + 1;
	h3 = h * 3;
	cy = renderer_h - 16;
	credit_text(
		gamestate,
		"Crow Buys Ketamine",
		cy
	);
#define br() \
	cy += h3;
#define text(t) \
	cy += h; \
	credit_text( \
		gamestate, \
		t, \
		cy \
	);
#define btext(t) \
	cy += h; \
	credit_text_right( \
		gamestate, \
		t, \
		cy \
	);
	br()
	text("Programming")
	text("Art")
	text("Sound")
	text("Design")
	btext("quou")
	br()
	text("Open Source Software")
	btext("GIMP")
	btext("GCC")
	btext("Microui")
	br()
	text("More stuff")
	btext("quou.xyz")
	br()
	text("Thanks for Playing!")
	cy = credit_scroll(gamestate, cy);
	if (
		cy < 0 &&
		gamestate->type != gamestate_type_transition
	) {
		transition_level(
			gamestate,
			"",
			world,
			gamestate_type_menu
		);
	}
	render_border();
}
