#include <string.h>

#include "cfgparse.h"
#include "cutparse.h"
#include "cutscene.h"
#include "memory.h"
#include "plat.h"
#include "sprite.h"

Cut_Cmd* alloc_cmd(
	Cutscene* s,
	int size
) {
	Cut_Cmd* c;
	size = (size + 7) & -8;
	if (s->buf_size + size > cutparse_buffer_size) {
		print_err("No more space for cutscene commands.");
		pbreak(error_out_of_memory);
		return 0;
	}
	c = (Cut_Cmd*)&s->buf[s->buf_size];
	memset(c, 0, c->size);
	c->size = size;
	s->buf_size += size;
	return c;
}

static void process_cmd_moveto(
	const cfg_Object* obj,
	Cutscene* s,
	int frame
) {
	Cut_Cmd_Moveto* cmd;
	int size;
	size = sizeof *cmd;
	cmd = (Cut_Cmd_Moveto*)alloc_cmd(s, size);
	cmd->cmd.type = cut_cmd_moveto;
	cmd->cmd.frame = frame;
	strncpy(
		cmd->handle,
		find_string_default(obj, "handle", "player"),
		cutscene_handle_size
	);
	cmd->handle[cutscene_handle_size - 1] = 0;
	cmd->x = find_int_default(obj, "x", 0);
	cmd->y = find_int_default(obj, "y", 0);
	cmd->speed = find_int_default(obj, "speed", 0);
}

static void process_cmd_dialog(
	const cfg_Object* obj,
	Cutscene* s,
	int frame
) {
	const char* txt;
	int len, size;
	Cut_Cmd_Dialog* cmd;
	txt = find_string_default(obj, "say", "<nothing>");
	len = strlen(txt);
	size = sizeof *cmd + len + 1;
	cmd = (Cut_Cmd_Dialog*)alloc_cmd(s, size);
	cmd->cmd.type = cut_cmd_dialog;
	cmd->cmd.frame = frame;
	cmd->face = get_sprite_id(
		find_string_default(obj, "face", "sprite_face_null")
	);
	strcpy((char*)(cmd + 1), txt); 
}

static void process_cmd(
	const cfg_Object* obj,
	Cutscene* s
) {
	const cfg_Prop* pframe;
	int frame;
	pframe = find_prop_of(obj, "frame", cfg_type_int);
	frame = pframe->as.num;
#define x(n, com) \
	if (!strcmp(#n, obj->name)) { \
		process_cmd_ ## n(obj, s, frame); \
		return; \
	}
	cut_cmd_type_xmacro()
#undef x
}

static void process_cut(
	const cfg_Object* obj,
	Cutscene* s
) {
	int frame;
	frame = find_int_default(obj, "endframe", 0);
	s->endframe = frame;
}

void cutscene_load_cfg(
	Cutscene* s,
	const cfg_Object* obj,
	Arena* arena
) {
	s->buf = arena_alloc(arena, cutparse_buffer_size);
	while (obj) {
		if (!strcmp("cutscene", obj->name))
			process_cut(obj, s);
		else
			process_cmd(obj, s);
		obj = obj->next;
	}
}
