#ifndef cutscene_h
#define cutscene_h

#include "config.h"
#include "entity.h"
#include "memory.h"

struct World;

#define cut_cmd_type_xmacro() \
	x(moveto, ctype_position) \
	x(dialog, 0)

typedef enum {
#define x(name, com) \
	cut_cmd_ ## name,
	cut_cmd_type_xmacro()
#undef x
	cut_cmd_type_count
} Cut_Cmd_Type;

typedef struct Cut_Cmd {
	Cut_Cmd_Type type;
	int size;
	int frame;
} Cut_Cmd;

typedef struct {
	Cut_Cmd cmd;
	char handle[cutscene_handle_size];
	int x, y;
	int speed;
} Cut_Cmd_Moveto;

typedef struct {
	Cut_Cmd cmd;
	int face;
} Cut_Cmd_Dialog;

typedef struct {
	const char* handle;
	Entity e;
} Cut_Entity;

typedef struct Cutscene {
	int endframe;
	int typewriter;
	char* buf;
	Cut_Cmd* cmd;
	int buf_size, frame;
	Cut_Cmd* run[cutscene_max_running];
	int running, paused;
} Cutscene;

const char* get_cut_cmd_type_name(
	Cut_Cmd_Type t
);
Cut_Cmd_Type get_cut_cmd_type_from_name(
	const char* name
);

void init_cutscene(Cutscene* s);
void load_cutscene(
	Cutscene* s,
	const char* name,
	Arena* arena
);
void render_cutscene(Cutscene* s);
void play_cutscene(Cutscene* s);
void play_cmd(Cutscene* s, Cut_Cmd* cmd);
void update_cutscene(Cutscene* s, struct World* world);
int cutscene_done(Cutscene* s);

#endif
