#include "asset.h"
#include "cutscene.h"
#include "gamestate.h"
#include "plat.h"
#include "rcache.h"
#include "render.h"
#include "sound.h"
#include "sprite_system.h"
#include "systems.h"
#include "world.h"

void gamestate_cutscene_init(
	Gamestate* gamestate,
	World* world
) {
	(void)world;
	gamestate->cutscene = arena_alloc(
		&gamestate->arena,
		sizeof *gamestate->cutscene
	);
	load_cutscene(
		gamestate->cutscene,
		gamestate->trans,
		&gamestate->arena
	);
	play_cutscene(gamestate->cutscene);
	play_song(0);
}

void gamestate_cutscene_update(
	Gamestate* gamestate,
	World* world
) {
	update_cutscene(gamestate->cutscene, world);
	animation_system(world);
	if (cutscene_done(gamestate->cutscene)) {
		play_song(1);
		switch_gamestate(
			gamestate,
			world,
			gamestate_type_general
		);
	}
}

void gamestate_cutscene_render(
	Gamestate* gamestate,
	World* world
) {
	Map* map;
	(void)gamestate;
	map = &world->map;
	render_map_background(map);
	render_map_layer(map, &map->layers[map_layer_background]);
	sprite_system(gamestate->spr_system, world);
	render_map_layer(map, &map->layers[map_layer_midground]);
	render_map_blood(map);
	render_map_layer(map, &map->layers[map_layer_foreground]);
	render_cutscene(gamestate->cutscene);
}
