#include "gamestate.h"
#include "memory.h"
#include "sprite_system.h"

void init_gamestate(Gamestate* gamestate) {
	gamestate->spr_system = stack_alloc(
		sizeof(Sprite_System)
	);
	gamestate->mem = galloc(gamestate_memory);
}

void switch_gamestate(
	Gamestate* gamestate,
	struct World* world,
	Gamestate_Type type
) {
	gamestate->prev = gamestate->type;
	gamestate->type = type;
	init_arena(
		&gamestate->arena,
		gamestate->mem,
		gamestate_memory
	);
	switch (type) {
#define x(n) \
	case gamestate_type_ ## n: \
		gamestate_ ## n ## _init(gamestate, world); \
		break;
	gamestate_types_xmacro()
	default: break;
#undef x
	}
}

void update_gamestate(
	Gamestate* gamestate,
	struct World* world
) {
	switch (gamestate->type) {
#define x(n) \
	case gamestate_type_ ## n: \
		gamestate_ ## n ## _update(gamestate, world); \
		gamestate_ ## n ## _render(gamestate, world); \
		break;
	gamestate_types_xmacro()
	default: break;
#undef x
	}
}

void render_gamestate_t(
	Gamestate* gamestate,
	struct World* world,
	Gamestate_Type type
) {
	switch (type) {
#define x(n) \
	case gamestate_type_ ## n: \
		gamestate_ ## n ## _render(gamestate, world); \
		break;
	gamestate_types_xmacro()
	default: break;
#undef x
	}
}

void init_gamestate_t(
	Gamestate* gamestate,
	struct World* world,
	Gamestate_Type type
) {
	switch (type) {
#define x(n) \
	case gamestate_type_ ## n: \
		gamestate_ ## n ## _init(gamestate, world); \
		break;
	gamestate_types_xmacro()
	default: break;
#undef x
	}
}
