#ifndef gamestate_h
#define gamestate_h

#include "memory.h"

struct Editor;
struct Gamestate;
struct Sprite_System;
struct World;
struct Menu;
struct Cutscene;

#define gamestate_types_xmacro() \
	x(general) \
	x(transition) \
	x(gameover) \
	x(menu) \
	x(cutscene) \
	x(pause) \
	x(credit)

typedef enum {
	gamestate_type_invalid,
#define x(n) \
	gamestate_type_ ## n,
	gamestate_types_xmacro()
#undef x
	gamestate_type_count
} Gamestate_Type;

typedef void (*Gamestate_Init)(
	struct Gamestate* gamestate,
	struct World* world
);
typedef void (*Gamestate_Update)(
	struct Gamestate* gamestate,
	struct World* world
);
typedef void (*Gamestate_Render)(
	struct Gamestate* gamestate,
	struct World* world
);

typedef struct Gamestate {
	Gamestate_Type type, prev;
	void* mem;
	Arena arena;
	struct Sprite_System* spr_system;
	struct Editor* editor;
	struct Menu* menu;
	struct Cutscene* cutscene;
	int tanim;
	int frame;
	int xscroll, yscroll;
	char trans[28];
	Gamestate_Type tto;
	int df;
} Gamestate;

void init_gamestate(Gamestate* gamestate);
void switch_gamestate(
	Gamestate* gamestate,
	struct World* world,
	Gamestate_Type type
);
void update_gamestate(
	Gamestate* gamestate,
	struct World* world
);
void render_gamestate(
	Gamestate* gamestate,
	struct World* world
);

#define x(n) \
	void gamestate_ ## n ## _init  (Gamestate*, struct World*); \
	void gamestate_ ## n ## _update(Gamestate*, struct World*); \
	void gamestate_ ## n ## _render(Gamestate*, struct World*);
	gamestate_types_xmacro()
#undef x

void transition_level(
	Gamestate* gamestate,
	const char* name,
	struct World* world,
	Gamestate_Type gs
);

void transition_load(
	Gamestate* gamestate,
	struct World* world
);

void init_gamestate_t(
	Gamestate* gamestate,
	struct World* world,
	Gamestate_Type type
);
void render_gamestate_t(
	Gamestate* gamestate,
	struct World* world,
	Gamestate_Type type
);

#endif
