#include <string.h>

#include "gamestate.h"
#include "memory.h"
#include "monster.h"
#include "plat.h"
#include "player.h"
#include "sprite_system.h"
#include "systems.h"
#include "world.h"

int pause_gameplay = 0;

void gamestate_general_init(
	Gamestate* gamestate,
	World* world
) {
	(void)gamestate;
	(void)world;
	gamestate->df = 0;
}

void gamestate_general_update(
	Gamestate* gamestate,
	World* world
) {
	(void)gamestate;
	if (!pause_gameplay) {
		if (!world->player.dead)
			update_player(&world->player, world);
		else {
			gamestate->df++;
			if (gamestate->df > player_dead_grace_frames) {
				transition_level(
					gamestate,
					"",
					world,
					gamestate_type_gameover
				);
			}
		}
		monster_system(world);
		boss_system(gamestate, world);
		lifetime_system(world);
		damage_system(world);
		follow_system(world);
		physics_system(world);
	}
	animation_system(world);
#if !enable_editor
	if (btn_just_pressed(btn_pause))
		switch_gamestate(
			gamestate,
			world,
			gamestate_type_pause
		);
#endif
}

void gamestate_general_render(
	Gamestate* gamestate,
	World* world
) {
	Map* map;
	(void)gamestate;
	map = &world->map;
	render_map_background(map);
	render_map_layer(map, &map->layers[map_layer_background]);
	sprite_system(gamestate->spr_system, world);
	render_map_layer(map, &map->layers[map_layer_midground]);
	render_map_blood(map);
	render_map_layer(map, &map->layers[map_layer_foreground]);
	boss_render_system(world);
}
