#include <string.h>

#include "levelloader.h"
#include "mapobject.h"
#include "memory.h"
#include "plat.h"
#include "rcache.h"
#include "systems.h"
#include "world.h"

typedef struct {
	unsigned char tile;
	unsigned short offset, size;
} Run;

void reset_world(World* world) {
	int i, s;
	char* c;
	s = sizeof *world;
	c = (char*)world;
	for (i = 0; i < s; i++, c++) {
		*c = 0;
	}
}

void preload_init(World* world, int w, int h) {
	unsigned s = world->seed;
	reset_world(world);
	init_world(world, s);
	init_map(&world->map, w, h);
}

void load_init(World* world, int spx, int spy) {
	Rectangle r;
	init_player(
		&world->player,
		world,
		spx,
		spy
	);
	cache_map(&world->map);
	r.x = 0;
	r.y = 0;
	r.w = renderer_w;
	r.h = renderer_h;
	rc_invalidate(&r);
	/* Kick the animations so that sprites show up. */
	animation_system(world);
}

void load_level(const char* name, World* world) {
	Arena arena;
	Asset_Data* data;
	Map* map;
	Map_Layer* layer;
	Map_Object* mo;
	Run* runs, * run, * nxt;
	World_Persistent p;
	unsigned char* buf;
	int ptr;
	int lc, w, h, spx, spy, index, i, j, k, l, m, mo_count;
	int rc, bs, rcm1, e;
#define read(i, type) do { \
		i = *(type*)&data->buffer[ptr];\
		ptr += sizeof(type); \
	} while (0)
	init_arena(
		&arena,
		galloc(level_loader_memory + 1024 * 2),
		level_loader_memory
	);
	data = read_asset_data(name, &arena);
	if (!data) {
		pbreak(error_invalid_asset);
	}
	if (memcmp(data->buffer, "LEVL", 4)) {
		print_err("Invalid level file.\n");
		pbreak(error_invalid_asset);
	}
	ptr = 4;
	ptr += 28; /* Skip the name for now */
	read(lc,  int);
	read(w,   int);
	read(h,   int);
	read(spx, int);
	read(spy, int);

#define decomp(dst) \
 	read(rc, int); \
 	read(bs, int); \
 	runs = (Run*)&data->buffer[ptr]; \
 	ptr += rc * sizeof *runs; \
 	buf = (unsigned char*)&data->buffer[ptr]; \
 	ptr += bs; \
 	k = l = 0; \
 	rcm1 = rc - 1; \
	for (j = 0; j < runs->offset; j++) \
		dst[k++] = buf[l++]; \
	for (j = 0; j < rcm1; j++) { \
		run = &runs[j]; \
		nxt = &runs[j + 1]; \
		for (m = 0; m < run->size; m++) \
			dst[k++] = run->tile; \
		e = nxt->offset - (run->offset + run->size); \
		for (m = 0; m < e; m++) \
			dst[k++] = buf[l++]; \
	} \
	run = &runs[j]; \
	for (j = 0; j < run->size; j++) \
		dst[k++] = run->tile; \

	p = world->persistent;
	preload_init(world, w, h);
	map = &world->map;
	for (i = 0; i < lc; i++) {
		read(index, int);
		if (index >= 0) {
			layer = &map->layers[index];
			decomp(layer->tiles);
		} else {
			decomp(map->colliders)
		}
	}
	read(mo_count, int);
	for (i = 0; i < mo_count; i++) {
		mo = (Map_Object*)&data->buffer[ptr];
		ptr += get_map_object_size(mo->type);
		spawn_mapobject(world, mo);
	}
	load_init(world, spx, spy);
	world->persistent = p;
#undef read
	gfree(arena.buffer);
}
