#include "asset.h"
#include "debuginfo.h"
#include "editor.h"
#include "gamestate.h"
#include "interactable.h"
#include "levelloader.h"
#include "map.h"
#include "maths.h"
#include "memory.h"
#include "monster.h"
#include "plat.h"
#include "player.h"
#include "rcache.h"
#include "render.h"
#include "sound.h"
#include "sprite_system.h"
#include "systems.h"
#include "world.h"

#if enable_profiler
#include "gf_profiling.c"
#endif

World* world;
Gamestate gamestate;
Arena sound_arena;

void render_loading();

void prog_init(void* memory) {
	memory_init(memory);
	render_init();
	rc_init();
	render_loading();
	preload_assets();
	init_arena(
		&sound_arena,
		stack_alloc(sound_memory),
		sound_memory
	);
	sound_init(&sound_arena);

	seed_rng((int)(unsigned long)memory);

	world = stack_alloc(sizeof *world);
	init_world(world, (int)(unsigned long)memory);

	init_gamestate(&gamestate);
	switch_gamestate(
		&gamestate,
		world,
#if enable_editor || defined(force_level)
		gamestate_type_general
#elif force_credits
		gamestate_type_credit
#else
		gamestate_type_menu
#endif
	);

#ifdef force_level
	load_level(force_level, world);
#endif

#if enable_editor
	init_editor();
	ed_load_level(world, "intermediate/level1.txt");
#endif
}

#ifdef plat_win
void render_malware_warning(void) {
	const char* text = "Warning: Malware Detected!";
	const Colour pink  = { 0xff, 0x00, 0xff, 0xff };
	const Colour white = { 0xff, 0xff, 0xff, 0xff };
	const BM_Font* f;
	int w;
	static int col = 0;
	f = get_main_font();
	w = text_rect(f, text).w;
	rc_add_cmd_rfont_text_col(
		f,
		renderer_w / 2 - w / 2,
		3,
		text,
		col? pink: white
	);
	col = !col;
}
#endif

void prog_update() {
	update_gamestate(&gamestate, world);
#if enable_editor
	update_editor(world);
	render_editor();
#endif
#ifdef plat_win
	render_malware_warning();
#endif
#if DEBUG && !enable_editor
	render_debuginfo(world);
#endif
	rc_flush();
}

void prog_deinit() {
#if enable_editor
	deinit_editor();
#endif
}
