#ifndef map_h
#define map_h

#include "config.h"
#include "game_config.h"
#include "map_tiles.h"
#include "render.h"
#include "sprite.h"

typedef enum {
#define x(n, ox, oy) n,
	map_tiles_xmacro
#undef x
	map_tiles_count
} Map_Tile;

#define map_collider_xmacro \
	x(map_collider_null) \
	x(map_collider_block) \
	x(map_collider_mask)

#define get_collider_type(byte) \
	(byte >> 4)

typedef enum {
#define x(n) n,
	map_collider_xmacro
#undef x
	map_collider_count
} Map_Collider;

#define map_layers_xmacro \
	x(map_layer_foreground, "Foreground") \
	x(map_layer_midground, "Midground") \
	x(map_layer_background, "Background")

typedef enum {
#define x(t, n) \
	t,
	map_layers_xmacro
#undef x
	map_layer_count
} Map_Layer_ID;

typedef struct {
	unsigned short x, y;
	Sprite_ID sprite;
} Corpse;

typedef struct {
	unsigned char tiles[map_max_w * map_max_h];
	Colour cache[
		map_max_w *
		map_max_h *
		map_tile_size *
		map_tile_size
	];
} Map_Layer;

typedef struct {
	Map_Layer layers[map_layer_count];
	Colour background[
		map_max_w *
		map_max_h *
		map_tile_size *
		map_tile_size
	];
	Colour blood[
		map_max_w *
		map_max_h *
		map_tile_size *
		map_tile_size
	];
	unsigned char colliders[map_max_w * map_max_h];
	Corpse corpses[map_max_corpses];
	int w, h, corpse_count;
} Map;

void init_map(Map* map, int w, int h);
void cache_map(Map* map);
void map_add_corpse(
	Map* map,
	int x,
	int y,
	Sprite_ID sprite
);
void init_map_layer(Map* map, Map_Layer* layer);
void render_map_background(Map* map);
void render_map_layer(Map* map, Map_Layer* layer);
void render_map_blood(Map* map);
void cache_map_layer(Map* map, Map_Layer* layer);
void map_add_blood(
	Map* map,
	int x,
	int y,
	Asset_ID bitmap,
	const Rectangle* rect
);

#endif
