#ifndef mapobject_h
#define mapobject_h

#include "cfgparse.h"
#include "entity.h"
#include "item.h"

struct World;
struct cfg_Object;

#define get_map_object_params(mo) ((void*)(&(mo)[1]))

#define map_objects_xmacro()\
	x(\
		prop, \
		create_prop, \
		edit_prop, \
		spawn_prop, \
		{ \
			int x; \
			int y; \
			int z; \
			int sprite; \
			int anim; \
			int random; \
			int collidable; \
			int destructable; \
		} \
	) \
	x(\
		cop, \
		create_cop, \
		edit_cop, \
		spawn_cop, \
		{ \
			int x; \
			int y; \
		} \
	) \
	x(\
		killzone, \
		create_killzone, \
		edit_killzone, \
		spawn_killzone, \
		{ \
			int sprite; \
			int x; \
			int y; \
			int ox; \
			int oy; \
			int w; \
			int h; \
		} \
	) \
	x(\
		conttrig, \
		create_conttrig, \
		edit_conttrig, \
		spawn_conttrig, \
		{ \
			char level[28]; \
			int x; \
			int y; \
			int w; \
			int h; \
		} \
	) \
	x( \
		gamesaver, \
		create_gamesaver, \
		edit_gamesaver, \
		spawn_gamesaver, \
		{ \
			int x; \
			int y; \
			int w; \
			int h; \
		} \
	) \
	x( \
		pickup, \
		create_pickup, \
		edit_pickup, \
		spawn_pickup, \
		{ \
			int x; \
			int y; \
			Item item; \
		} \
	) \
	x( \
		cuttrig, \
		create_cuttrig, \
		edit_cuttrig, \
		spawn_cuttrig, \
		{ \
			char cut[28]; \
			int x; \
			int y; \
			int w; \
			int h; \
		} \
	) \
	x( \
		boss, \
		create_boss, \
		edit_boss, \
		spawn_boss, \
		{ \
			int type; \
			int x; \
			int y; \
		} \
	)

typedef enum {
#define x(type, factory, editor, spawner, params) \
	map_object_ ## type,
	map_objects_xmacro()
#undef x
	map_object_type_count
} Map_Object_Type;

typedef struct {
	Map_Object_Type type;
} Map_Object;

typedef Map_Object* (*Map_Object_Factory)(
	const cfg_Object* obj,
	Arena* arena
);

typedef Entity (*Map_Object_Spawner)(
	struct World*,
	const Map_Object*
);

typedef void (*Map_Object_Editor)(
	struct World*,
	Map_Object*,
	Entity e
);

#define x(type, factory, editor, spawner, params) \
	Map_Object* create_ ## type( \
		const cfg_Object* obj, \
		Arena* arena \
	); \
	Entity spawn_ ## type( \
		struct World* world, \
		const Map_Object* obj \
	); \
	void edit_ ## type( \
		struct World* world, \
		Map_Object* obj, \
		Entity e \
	); \
	typedef struct \
		params \
	type ## _paramstruct;
map_objects_xmacro()
#undef x

Map_Object* create_map_object(
	const char* name,
	const cfg_Object* obj,
	Arena* arena
);
Map_Object_Type get_map_object_type(const char* name);
int get_map_object_size(Map_Object_Type type);
Entity spawn_mapobject(
	struct World* world,
	const Map_Object* obj
);
const char* get_map_object_name(Map_Object_Type type);

#endif
