#include "boss.h"
#include "mapobject.h"
#include "world.h"

static const Map_Object_Spawner mo_spawners[] = {
#define x(type, factory, editor, spawner, params) \
	spawner,
map_objects_xmacro()
#undef x
};

Entity spawn_mapobject(
	World* world,
	const Map_Object* obj
) {
	return mo_spawners[obj->type](world, obj);
}

Entity spawn_prop(
	World* world,
	const Map_Object* obj
) {
	Entity e;
	int id;
	const prop_paramstruct* params;
	CSprite* sprite;
	CPosition* pos;
	CAnimated* anim;
	CCollider* col;
	CBody* body;
	CVelocity* vel;
	CDamageable* dam;
	e = new_entity(world);
	id = entity_index(e);
	params = (void*)(&obj[1]);
	add_components(
		world,
		e,
		ctype_position |
		ctype_sprite
	);
	pos = &world->positions[id];
	sprite = &world->sprites[id];
	pos->x = params->x;
	pos->y = params->y;
	if (params->anim) {
		add_components(
			world,
			e,
			ctype_animated
		);
		anim = &world->animated[id];
		if (params->random) {
			anim->frame = rng() %
				get_animation(params->sprite)->frame_count;
		} else
			anim->frame = 0;
		anim->timer = 0;
		anim->id = params->sprite;
	} else
		init_csprite(sprite, params->sprite);
	if (params->collidable || params->destructable) {
		add_components(
			world,
			e,
			ctype_collider
		);
		col = &world->colliders[id];
		col->x = 0;
		col->y = 0;
		col->w = sprite->rect.w << fbits;
		col->h = sprite->rect.h << fbits;
	}
	if (params->collidable)
		add_tags(
			world,
			e,
			tag_collidable
		);
	if (params->destructable) {
		add_components(
			world,
			e,
			ctype_body |
			ctype_velocity |
			ctype_damageable
		);
		add_tags(
			world,
			e,
			tag_enemy | 
			tag_killable
		);
		body = &world->bodies[id];
		dam = &world->damageables[id];
		vel = &world->velocities[id];
		body->grounded = 0;
		vel->x = 0;
		vel->y = 0;
		dam->hp = 1;
	}
	sprite->z = params->z;
	return e;
}

Entity spawn_cop(
	World* world,
	const Map_Object* obj
) {
	Entity e;
	int id;
	const cop_paramstruct* params;
	CSprite* sprite;
	CPosition* pos;
	CMonster* mons;
	CBody* body;
	CCollider* col;
	CVelocity* vel;
	CDamageable* dam;
	e = new_entity(world);
	id = entity_index(e);
	params = (void*)(&obj[1]);
	add_components(
		world,
		e,
		ctype_position |
		ctype_sprite   |
		ctype_monster  |
		ctype_cop      |
		ctype_body     |
		ctype_collider |
		ctype_velocity |
		ctype_damageable
	);
	add_tags(
		world,
		e,
		tag_enemy |
		tag_killable |
		tag_bio
	);
	pos = &world->positions[id];
	sprite = &world->sprites[id];
	mons = &world->monsters[id];
	body = &world->bodies[id];
	col = &world->colliders[id];
	vel = &world->velocities[id];
	dam = &world->damageables[id];
	dam->hp = 1;
	vel->x = 0;
	vel->y = 0;
	col->x = 0  << fbits;
	col->y = 0  << fbits;
	col->w = 16 << fbits;
	col->h = 16 << fbits;
	body->grounded = 1;
	pos->x = params->x;
	pos->y = params->y;
	mons->timer = 0;
	init_csprite(sprite, sprite_cop_left);
	return e;
}

Entity spawn_killzone(
	World* world,
	const Map_Object* obj
) {
	Entity e;
	int id;
	const killzone_paramstruct* params;
	CSprite* sprite;
	CPosition* pos;
	CCollider* col;
	CVelocity* vel;
	CBody* body;
	e = new_entity(world);
	id = entity_index(e);
	params = (void*)(&obj[1]);
	add_components(
		world,
		e,
		ctype_position |
		ctype_sprite |
		ctype_collider |
		ctype_body |
		ctype_velocity
	);
	add_tags(
		world,
		e,
		tag_enemy |
		tag_killzone
	);
	pos = &world->positions[id];
	sprite = &world->sprites[id];
	col = &world->colliders[id];
	body = &world->bodies[id];
	vel = &world->velocities[id];
	pos->x = params->x;
	pos->y = params->y;
	col->x = params->ox;
	col->y = params->oy;
	col->w = params->w;
	col->h = params->h;
	vel->x = 0;
	vel->y = 0;
	body->grounded = 0;
	init_csprite(sprite, params->sprite);
	return e;
}

Entity spawn_conttrig(
	World* world,
	const Map_Object* obj
) {
	Entity e;
	int id;
	const conttrig_paramstruct* params;
	const char* s;
	char* d;
	CPosition* pos;
	CCollider* col;
	CBody* body;
	CVelocity* vel;
	CConttrig* conttrig;
	e = new_entity(world);
	id = entity_index(e);
	params = (void*)(&obj[1]);
	add_components(
		world,
		e,
		ctype_position |
		ctype_collider |
		ctype_body |
		ctype_velocity |
		ctype_conttrig
	);
	pos = &world->positions[id];
	col = &world->colliders[id];
	body = &world->bodies[id];
	vel = &world->velocities[id];
	conttrig = &world->conttrigs[id];
	pos->x = params->x;
	pos->y = params->y;
	vel->x = 0;
	vel->y = 0;
	body->grounded = 0;
	col->x = 0;
	col->y = 0;
	col->w = params->w;
	col->h = params->h;
	for (
		s = params->level, d = conttrig->level;
		*s;
		s++, d++
	) *d = *s;
	*d = 0;
	return e;
}

Entity spawn_gamesaver(
	World* world,
	const Map_Object* obj
) {
	Entity e;
	int id;
	const gamesaver_paramstruct* params;
	CPosition* pos;
	CCollider* col;
	CBody* body;
	CVelocity* vel;
	CSprite* sprite;
	e = new_entity(world);
	id = entity_index(e);
	params = (void*)(&obj[1]);
	add_components(
		world,
		e,
		ctype_position |
		ctype_collider |
		ctype_body |
		ctype_velocity |
		ctype_sprite
	);
	add_tags(
		world,
		e,
		tag_savegame
	);
	pos = &world->positions[id];
	col = &world->colliders[id];
	body = &world->bodies[id];
	vel = &world->velocities[id];
	sprite = &world->sprites[id];
	pos->x = params->x;
	pos->y = params->y;
	vel->x = 0;
	vel->y = 0;
	body->grounded = 0;
	col->x = 0;
	col->y = 0;
	col->w = params->w;
	col->h = params->h;
	init_csprite(sprite, sprite_floppy);
	return e;
}

Entity spawn_pickup(
	World* world,
	const Map_Object* obj
) {
	Entity e;
	int id;
	const pickup_paramstruct* params;
	CPosition* pos;
	CCollider* col;
	CBody* body;
	CVelocity* vel;
	CSprite* sprite;
	CPickup* pickup;
	e = new_entity(world);
	id = entity_index(e);
	params = (void*)(&obj[1]);
	add_components(
		world,
		e,
		ctype_position |
		ctype_collider |
		ctype_body |
		ctype_velocity |
		ctype_sprite |
		ctype_pickup
	);
	add_tags(
		world,
		e,
		tag_trigger
	);
	pos = &world->positions[id];
	col = &world->colliders[id];
	body = &world->bodies[id];
	vel = &world->velocities[id];
	sprite = &world->sprites[id];
	pickup = &world->pickups[id];
	init_csprite(sprite, get_item_sprite(params->item));
	pos->x = params->x;
	pos->y = params->y;
	vel->x = 0;
	vel->y = 0;
	body->grounded = 0;
	col->x = 0;
	col->y = 0;
	col->w = sprite->rect.w << fbits;
	col->h = sprite->rect.h << fbits;
	pickup->item = params->item;
	return e;
}

Entity spawn_cuttrig(
	World* world,
	const Map_Object* obj
) {
	Entity e;
	int id;
	const char* s;
	char* d;
	const cuttrig_paramstruct* params;
	CPosition* pos;
	CCollider* col;
	CBody* body;
	CVelocity* vel;
	CCuttrig* ct;
	e = new_entity(world);
	id = entity_index(e);
	params = (void*)(&obj[1]);
	add_components(
		world,
		e,
		ctype_position |
		ctype_collider |
		ctype_body |
		ctype_velocity |
		ctype_cuttrig
	);
	pos = &world->positions[id];
	col = &world->colliders[id];
	body = &world->bodies[id];
	vel = &world->velocities[id];
	ct = &world->cuttrigs[id];
	pos->x = params->x;
	pos->y = params->y;
	vel->x = 0;
	vel->y = 0;
	body->grounded = 0;
	col->x = 0;
	col->y = 0;
	col->w = params->w;
	col->h = params->h;
	for (
		s = params->cut, d = ct->cut;
		*s;
		s++, d++
	) *d = *s;
	*d = 0;
	return e;
}

Entity spawn_boss(
	World* world,
	const Map_Object* obj
) {
	const boss_paramstruct* params;
	params = (void*)(&obj[1]);
	return new_boss(
		params->type,
		world,
		params->x,
		params->y
	);
}
