#include "asset.h"
#include "menu.h"
#include "plat.h"
#include "rcache.h"
#include "render.h"
#include "sprite.h"

void init_menu(Menu* menu) {
	menu->item_count = 0;
	menu->selected = 0;
}

void menu_add(
	Menu* menu,
	const char* label,
	Menu_Callback clbk
) {
	Menu_Item* i;
	int id;
	char* d;
	const char* s;
#ifdef DEBUG
	if (menu->item_count > max_menu_items) {
		print_err("Out of space for new menu items.\n");
		pbreak(error_out_of_memory);
	}
#endif
	id = menu->item_count++;
	i = &menu->items[id];
	for (d = i->label, s = label; *s; d++, s++)
		*d = *s;
	*d = 0;
	i->clbk = clbk;
}

void menu_select(Menu* menu, int id) {
	menu->selected = id;
}

void menu_update(Menu* menu) {
	if (btn_just_pressed(btn_up))
		if (--menu->selected <= 0)
			menu->selected = 0;
	if (btn_just_pressed(btn_down))
		if (++menu->selected >= menu->item_count)
			menu->selected = menu->item_count - 1;
	if (btn_just_pressed(btn_jump))
		menu->items[menu->selected].clbk(
			menu,
			menu->selected
		);
}

void menu_draw(const Menu* menu) {
	int i, x, y, w;
	const BM_Font* font;
	const Menu_Item* item;
	const Sprite* selector;
	const Bitmap* bsele;
	Rectangle r;
	font = get_main_font();
	selector = get_sprite(sprite_bomb);
	bsele = get_bitmap(selector->bitmap);
	w = renderer_w;
	y = renderer_h / 2 - font->height / 2;
	for (i = 0; i < menu->item_count; i++) {
		item = &menu->items[i];
		r = text_rect(font, item->label);
		x = w / 2 - r.w / 2;
		if (i == menu->selected) {
			rc_add_cmd_bmp(
				bsele,
				x - selector->rect.w - 1,
				y,
				&selector->rect
			);
		}
		rc_add_cmd_rfont_text(
			font,
			x,
			y,
			item->label
		);
		y += font->height;
	}
}
