#include "asset.h"
#include "gamestate.h"
#include "menu.h"
#include "plat.h"
#include "rcache.h"
#include "render.h"
#include "sound.h"
#include "world.h"

static void on_new_select(Menu* menu, int id) {
	Gamestate* gamestate;
	World* world;
	(void)id;
	set_song(0);
	gamestate = menu->uptr[0];
	world = menu->uptr[1];
	transition_level(
		gamestate,
		"level1.lvl",
		world,
		gamestate_type_general
	);
}

static void on_load_select(Menu* menu, int id) {
	Gamestate* gamestate;
	World* world;
	(void)id;
	set_song(0);
	gamestate = menu->uptr[0];
	world = menu->uptr[1];
	if (!file_exists(savefile_name)) {
		on_new_select(menu, id);
		return;
	}
	transition_load(gamestate, world);
}

void gamestate_menu_init(
	Gamestate* gamestate,
	World* world
) {
	gamestate->menu = arena_alloc(
		&gamestate->arena,
		sizeof *gamestate->menu
	);
	init_menu(gamestate->menu);
	menu_add(gamestate->menu, "New",  on_new_select);
	menu_add(gamestate->menu, "Load", on_load_select);
	gamestate->menu->uptr[0] = gamestate;
	gamestate->menu->uptr[1] = world;
	if (file_exists(savefile_name)) {
		menu_select(gamestate->menu, 1);
	}
	set_song("menu.pcm");
	play_song(1);
}

void gamestate_menu_update(
	Gamestate* gamestate,
	World* world
) {
	(void)world;
	menu_update(gamestate->menu);
}

void render_border(void) {
	const int ts = 16;
	int x, y, ex, ey, i;
	const Sprite* s;
	const Bitmap* b;
	ex = renderer_w / ts - 1;
	ey = renderer_h / ts - 1;
	i = 0;
	x = 0;
#define draw { \
	s = get_sprite(sprite_border_blood + (i++) % 2); \
	b = get_bitmap(s->bitmap); \
	rc_add_cmd_bmp( \
		b, \
		x * ts, \
		y * ts, \
		&s->rect \
	); \
}
	for (x = 0, y = 0; y <= ey; y++)
		draw
	for (x = ex, y = 0; y <= ey; y++)
		draw
	for (x = 1, y = 0; x < ex; x++)
		draw
	for (x = 1, y = ey; x < ex; x++)
		draw
#undef draw
}

void gamestate_menu_render(
	Gamestate* gamestate,
	World* world
) {
	Rectangle r;
	const Colour grey = { 200, 200, 200, 255 };
	const Colour blue = { 200, 200, 255, 255 };
	const char* text = "Crow Buys Ketamine";
	const BM_Font* font;
	const Fixed_Font* sfont;
	(void)world;
	font = get_main_font();
	sfont = get_smol_font();
	menu_draw(gamestate->menu);
	r = text_rect(font, text);
	rc_add_cmd_rfont_text(
		font,
		renderer_w / 2 - r.w / 2,
		80,
		text
	);
	rc_add_cmd_rffont_text_col(
		sfont,
		renderer_w / 2 - 30,
		150,
		"made by quou",
		grey
	);
	rc_add_cmd_rffont_text_col(
		sfont,
		renderer_w / 2 - 20,
		155,
		"quou.xyz",
		blue
	);
	render_border();
}
