#include "fx.h"
#include "game_config.h"
#include "gamestate.h"
#include "memory.h"
#include "plat.h"
#include "profile.h"
#include "render.h"
#include "tags.h"
#include "world.h"

/* TODO: Don't do this. */
extern Gamestate gamestate;

static void handle_cuttrig(
	World* world,
	Entity e
) {
	int id;
	const char* s;
	char* d;
	CCuttrig* t;
	id = entity_index(e);
	t = &world->cuttrigs[id];
	s = t->cut;
	for (d = gamestate.trans; *s; d++, s++)
		*d = *s;
	*d = 0;
	switch_gamestate(
		&gamestate,
		world,
		gamestate_type_cutscene
	);
	destroy_entity(world, e);
}

extern void kill_killable(
	World* world,
	Entity e
);

static void handle_bodies(
	World* world,
	Entity a,
	Entity b
) {
	Tags at, bt;
	Com_Bitset ac, bc;
	int ai, bi;
	CPosition* pos;
	CSprite* spr;
	CDamageable* dam;
	ai = entity_index(a);
	bi = entity_index(b);
	at = world->tags[ai];
	bt = world->tags[bi];
	ac = world->bitset[ai];
	bc = world->bitset[bi];
	if (at & tag_killzone && bt & tag_killable)
		kill_killable(world, b);
	if (at & tag_killable && bt & tag_killzone)
		kill_killable(world, a);
	if (
		(at & tag_bullet && bc & ctype_damageable) &&
		((at & tag_enemy) != (bt & tag_enemy))
	) {
		dam = &world->damageables[bi];
		if (at & tag_bomb && ac & ctype_sprite) {
			pos = &world->positions[ai];
			spr = &world->sprites[ai];
			new_explosion(
				world,
				pos->x + ((spr->rect.w << fbits) / 2),
				pos->y + ((spr->rect.h << fbits) / 2)
			);
			destroy_entity(world, a);
		}
		dam->hp--;
		dam->hurting = 10;
	}
	if (
		(ac & ctype_damageable && bt & tag_bullet) &&
		((at & tag_enemy) != (bt & tag_enemy))
	) {
		dam = &world->damageables[ai];
		if (bt & tag_bomb && bc & ctype_sprite) {
			pos = &world->positions[bi];
			spr = &world->sprites[bi];
			new_explosion(
				world,
				pos->x + ((spr->rect.w << fbits) / 2),
				pos->y + ((spr->rect.h << fbits) / 2)
			);
			destroy_entity(world, b);
		}
		dam->hp--;
		dam->hurting = 10;
	}
	if (ac & ctype_conttrig && bt & tag_player)
		transition_level(
			&gamestate,
			world->conttrigs[ai].level,
			world,
			gamestate_type_general
		);
	if (at & tag_player && bc & ctype_conttrig)
		transition_level(
			&gamestate,
			world->conttrigs[bi].level,
			world,
			gamestate_type_general
		);
	if (ac & ctype_cuttrig && bt & tag_player)
		handle_cuttrig(world, a);
	if (at & tag_player && bc & ctype_cuttrig)
		handle_cuttrig(world, b);
	if (at & tag_savegame && bt & tag_player) {
		destroy_entity(world, a);
		save_game(world, savefile_name);
	}
	if (at & tag_player && bt & tag_savegame) {
		destroy_entity(world, b);
		save_game(world, savefile_name);
	}
	if (ac & ctype_pickup && bt & tag_player) {
		pickup_item(world, a);
	}
	if (bc & ctype_pickup && at & tag_player) {
		pickup_item(world, b);
	}
}

static void handle_stat_collide(
	World* world,
	Entity e
) {
	Com_Bitset c, m;
	Tags t;
	CPosition* pos;
	CSprite* sprite;
	const Rectangle* ar;
	const CImpact* impact;
	int i;
	i = entity_index(e);
	c = world->bitset[i];
	t = world->tags[i];
	m = (ctype_impact | ctype_sprite);
	if ((c & m) == m) {
		pos =    &world->positions[i];
		sprite = &world->sprites[i];
		impact = &world->impacts[i];
		ar = &get_animation(impact->anim)->frames[0];
		new_impact(
			world,
			pos->x +
				(sprite->rect.w << fbits) / 2 -
				(ar->w << fbits) / 2,
			pos->y +
				(sprite->rect.h << fbits) / 2 -
				(ar->h << fbits) / 2,
			impact->anim
		);
	}
	if (t & tag_destroy_collision) {
		destroy_entity(world, e);
		return;
	}
	if (c & ctype_blood) {
		pos = &world->positions[i];
		sprite = &world->sprites[i];
		map_add_blood(
			&world->map,
			pos->x >> fbits,
			pos->y >> fbits,
			sprite->asset,
			&sprite->rect
		);
		destroy_entity(world, e);
		return;
	}
	if (t & tag_bomb) {
		pos = &world->positions[i];
		sprite = &world->sprites[i];
		new_explosion(
			world,
			pos->x + ((sprite->rect.w << fbits) / 2),
			pos->y + ((sprite->rect.h << fbits) / 2)
		);
		destroy_entity(world, e);
		return;
	}
}

static void update_body_triggers(
	World* world,
	Entity e,
	const CCollider* col,
	const CPosition* pos,
	int i
) {
	Rectangle ar, br;
	int j;
	Com_Bitset mask, bits;
	const CCollider* colb;
	const CPosition* posb;
	ar.x = pos->x + col->x;
	ar.y = pos->y + col->y;
	ar.w = col->w;
	ar.h = col->h;
	mask = ctype_position | ctype_collider;
	for (j = i + 1; j < world->created_count; j++) {
		bits = world->bitset[j];
		if ((bits & mask) != mask) continue;
		colb = &world->colliders[j];
		posb = &world->positions[j];
		br.x = posb->x + colb->x;
		br.y = posb->y + colb->y;
		br.w = colb->w;
		br.h = colb->h;
		if (rects_overlap(&ar, &br))
			handle_bodies(world, e, world->entities[j]);
	}
}

static int solve_mask(
	unsigned char t,
	CPosition* pos,
	CVelocity* vel,
	const CCollider* col,
	const Rectangle* r,
	const Rectangle* tr,
	int* od
) {
	int mx, my, n, d = 0;
	const Bitmap* b = get_bitmap(asset_id_colmask);
	unsigned red = 0xff0000ff;
	unsigned* mp, p;
	mx = r->x + r->w / 2;
	my = (r->y + r->h) - f1;
	mx -= tr->x;
	my -= tr->y;
	mx >>= fbits;
	my >>= fbits;
	if (mx < 0 || mx >= map_tile_size)
		return 0;
	if ((n = my - map_tile_size) >= 0 && n < 4)
		my = map_tile_size - 1;
	mp = (void*)(b->pixels + (t & 0xf) * map_tile_size);
	if (my >= map_tile_size) return 0;
	if (my < 0 || vel->y < 0) goto up;
	p = mp[mx + my * b->w];
	if (p == red) {
		d = -1;
		my = map_tile_size - 1;
	} else {
up:
		my = (r->y - tr->y) >> fbits;
		if (my < 0) {
			if (my >= -4) my = 0;
			else          return 0;
		}
		if (my >= map_tile_size) return 0;
		p = mp[mx + my * b->w];
		if (p == red) d = 1;
		my = 0;
	}
	if (!d)                   return 0;
	if (d < 0 && vel->y <= 0) return 0;
	if (d > 0 && vel->y >= 0) return 0;
	for (
		;
			mp[mx + my * b->w] == red &&
			my >= 0 &&
			my < map_tile_size;
		my += d
	);
	if (d < 0) {
		pos->y = (tr->y - (col->y + col->h));
		pos->y += (my << fbits) + f1 * 2;
	} else {
		pos->y = (tr->y + tr->h) - col->y;
		pos->y -= ((map_tile_size - my) << fbits);
	}
	vel->y = 0;
	*od = d;
	return 1;
}

static void update_body_collision(
	World* world,
	Entity e,
	CBody* body,
	const CCollider* col,
	CPosition* pos,
	CVelocity* vel
) {
	int sx, sy, tx, ty, ex, ey, mx, my, f = 0;
	int hs = 0, cy, d;
	const CCollider* ccol;
	const CPosition* cpos;
	const Map* map;
	Map_Collider tc;
	Rectangle tr, rect;
	general_system_vars;
	profiler_begin("update_body_collision");
	map = &world->map;
	mx = map->w;
	my = map->h;
	rect.w = col->w;
	rect.h = col->h;
	body->grounded++;
#define ittiles(expr) \
	rect.x = pos->x + col->x; \
	rect.y = pos->y + col->y; \
	sx = (rect.x >> fbits) / map_tile_size; \
	sy = (rect.y >> fbits) / map_tile_size; \
	ex = ((rect.x + rect.w) >> fbits) / map_tile_size; \
	ey = ((rect.y + rect.h) >> fbits) / map_tile_size; \
	tr.w = map_tile_size << fbits; \
	tr.h = map_tile_size << fbits; \
	for (ty = sy; ty <= ey && ty < my; ty++) { \
		for (tx = sx; tx <= ex && tx < mx; tx++) { \
			tc = map->colliders[tx + ty * map->w]; \
			tr.x = (tx * map_tile_size) << fbits; \
			tr.y = (ty * map_tile_size) << fbits; \
			if (rects_overlap(&tr, &rect)) \
				expr \
		} \
	} \
	general_system( \
		ctype_collider | \
		ctype_position \
	) \
		if (~world->tags[i] & tag_collidable) \
			continue; \
		if (world->entities[i] == e) \
			continue; \
		cpos = &world->positions[i]; \
		ccol = &world->colliders[i]; \
		tr.x = cpos->x + ccol->x; \
		tr.y = cpos->y + ccol->y; \
		tr.w = ccol->w; \
		tr.h = ccol->h; \
		tc = map_collider_block << 4; \
		if (rects_overlap(&tr, &rect)) \
			expr \
	}
	if (vel->y) {
		pos->y += vel->y;
		ittiles({
			if (
				get_collider_type(tc) == map_collider_mask &&
				solve_mask(tc, pos, vel, col, &rect, &tr, &d)
			) {
				if (d < 0)
					body->grounded = 0;
				hs = 1;
				f = 1;
			}
		});
		if (hs) goto do_x;
		ittiles({
			if (get_collider_type(tc) == map_collider_block) {
				f = 1;
				if (vel->y > 0) {
					pos->y = tr.y - (col->y + col->h);
					vel->y = 0;
					body->grounded = 0;
				} else if (vel->y < 0) {
					pos->y = (tr.y + tr.h) - col->y;
					vel->y = 0;
				}
			}
		});
	}
do_x:
	if (vel->x) {
		pos->x += vel->x;
		if (hs) {
			/* Handle the case where the body is at the top
			 * of a ramp hitting into a block. */
			ittiles({
				if (get_collider_type(tc) == map_collider_block) {
					if (d < 0) cy = rect.y;
					else cy = rect.y + rect.h;
					if (vel->x < 0 && point_rect_overlap(
						&tr,
						rect.x,
						cy
					)) pos->x = (tr.x + tr.w) - col->x;
					if (vel->x > 0 && point_rect_overlap(
						&tr,
						rect.x + rect.w,
						cy
					)) pos->x = tr.x - (col->x + col->w);
				}
			})
			goto skip_x;
		}
		ittiles({
			if (get_collider_type(tc) == map_collider_block) {
				f = 1;
				if (vel->x > 0) {
					pos->x = tr.x - (col->x + col->w);
					vel->x = 0;
				} else if (vel->x < 0) {
					pos->x = (tr.x + tr.w) - col->x;
					vel->x = 0;
				}
			}
		});
	}
skip_x:
	if (f) {
		handle_stat_collide(world, e);
	}
	profiler_end();
}

static void update_collision(World* world) {
	general_system_vars
	CBody* body;
	CCollider* col;
	CPosition* pos;
	CVelocity* vel;
	CSprite* sprite;
	int w, h;
	profiler_begin("update_collision");
	general_system(
		ctype_body |
		ctype_velocity |
		ctype_collider |
		ctype_position
	)
		body = &world->bodies[i];
		vel = &world->velocities[i];
		col = &world->colliders[i];
		pos = &world->positions[i];
		update_body_collision(
			world,
			world->entities[i],
			body,
			col,
			pos,
			vel
		);
		update_body_triggers(
			world,
			world->entities[i],
			col,
			pos,
			i
		);
	}
	/* Destroy offscreen entities. */
	mask = ctype_position;
	for (i = 0; i < world->created_count; i++) {
		bits = world->bitset[i];
		if (~world->tags[i] & tag_destroy_offscreen)
			continue;
		if ((bits & mask) != mask) continue;
		w = h = 0;
		if (bits & ctype_sprite) {
			sprite = &world->sprites[i];
			w = sprite->rect.w;
			h = sprite->rect.h;
		} else if (bits & ctype_collider) {
			col = &world->colliders[i];
			w = col->w >> fbits;
			h = col->h >> fbits;
		}
		pos = &world->positions[i];
		if (
			pos->x < 0 ||
			pos->y < 0 ||
			(pos->x >> fbits) + w > renderer_w ||
			(pos->y >> fbits) + h > renderer_h
		)
			destroy_entity(world, world->entities[i]);
	}
	profiler_end();
}

void physics_system(
	World* world
) {
	general_system_vars
	Tags tags;
	CPosition* pos;
	CVelocity* vel;
	CFriction* fric;
	CBody* body;
	int famt;
	profiler_begin("physics_system");
	general_system(ctype_body | ctype_velocity | ctype_friction)
		body = &world->bodies[i];
		vel  = &world->velocities[i];
		fric = &world->frictions[i];
		famt = fric->amt;
		if (body->grounded < 3 || famt & 1) {
			vel->x = (vel->x << fbits) / (f1 + (famt >> 1));
		}
	}
	general_system(ctype_position | ctype_velocity)
		tags = world->tags[i];
		pos = &world->positions[i];
		vel = &world->velocities[i];
		if (~bits & ctype_body) {
			pos->x += vel->x;
			pos->y += vel->y;
		}
		if (tags & tag_gravity) {
			vel->y += main_gravity;
			if (vel->y > max_velocity)
				vel->y = max_velocity;
		}
	}
	update_collision(world);
	profiler_end();
}
