#ifndef plat_h
#define plat_h

#include "error.h"

typedef struct File File;

typedef enum {
	file_flags_write       = 1 << 0,
	file_flags_read        = 1 << 1,
	file_flags_read_write  = 1 << 2,
	file_flags_create      = 1 << 3
} File_Flags;

typedef enum {
	file_whence_begin,
	file_whence_end,
	file_whence_cur
} File_Whence;

File* file_open(const char* name, File_Flags flags);
void file_close(File* file);
void file_read(File* file, void* buffer, int size);
void file_write(File* file, const void* buffer, int size);
void file_seek(File* file, int offset, File_Whence whence);
int file_size(File* file);
int file_exists(const char* name);

void print(const char* fmt, ...);
void print_err(const char* fmt, ...);
void print_war(const char* fmt, ...);
void pbreak(Error code);

void plat_quit(void);
void plat_flush(void);

#if enable_mouse
int get_mouse_x();
int get_mouse_y();

typedef enum {
	mbtn_left,
	mbtn_middle,
	mbtn_right,
	mbtn_count
} MBtn;
int mbtn_pressed(MBtn btn);
int mbtn_just_pressed(MBtn btn);
int mbtn_just_released(MBtn btn);
int get_mscroll_x(void);
int get_mscroll_y(void);
#endif

unsigned long get_current_time();
void sleep_micro(unsigned long ns);

typedef enum {
	btn_unknown = 0,
	btn_left,
	btn_right,
	btn_up,
	btn_down,
	btn_shoot,
	btn_jump,
	btn_special,
	btn_inv,
	btn_pause,
	btn_pc_shift,
	btn_pc_ctrl,
	btn_pc_alt,
	btn_pc_backspace,
	btn_pc_return,
	btn_pc_f12,
	btn_count
} Btn;

int btn_pressed(Btn btn);
int btn_just_pressed(Btn btn);
int btn_just_released(Btn btn);
int any_btn_just_pressed(void);
const char* get_text_input(int* len);

int plat_fps(void);

void plat_present(
	int x,
	int y,
	int w,
	int h
);

/* internal */
typedef void* (*Int_Audio_Worker)(void*);
void int_plat_start_audio(Int_Audio_Worker w, void* p);
void int_plat_wait_audio(void);
void int_plat_lock_audio(void);
void int_plat_unlock_audio(void);

#endif
