#include <string.h>

#include "arms.h"
#include "fx.h"
#include "game_config.h"
#include "gamestate.h"
#include "maths.h"
#include "plat.h"
#include "player.h"
#include "world.h"

extern Gamestate gamestate;

void init_player(Player* player, World* world, int x, int y) {
	Entity e;
	int id;
	CAnimated* animated;
	CPosition* pos;
	CVelocity* vel;
	CCollider* col;
	CSprite* sprite;
	CFriction* fric;
	CBody* body;
	CFace* face;
	CCutscene* cut;
	CDamageable* dam;
	e = new_entity(world);
	id = entity_index(e);
	add_components(
		world,
		e,
		ctype_position |
		ctype_velocity |
		ctype_animated |
		ctype_sprite   |
		ctype_collider |
		ctype_body     |
		ctype_face     |
		ctype_friction |
		ctype_cutscene |
		ctype_damageable
	);
	add_tags(
		world,
		e,
		tag_gravity  |
		tag_dynamic  |
		tag_friend   |
		tag_killable |
		tag_player   |
		tag_bio
	);
	pos = &world->positions[id];
	animated = &world->animated[id];
	vel = &world->velocities[id];
	col = &world->colliders[id];
	sprite = &world->sprites[id];
	fric = &world->frictions[id];
	body = &world->bodies[id];
	face = &world->faces[id];
	cut = &world->cutscenes[id];
	dam = &world->damageables[id];
	pos->x = x;
	pos->y = y;
	vel->x = 0;
	vel->y = 0;
	col->x = 0 << fbits;
	col->y = 0 << fbits;
	col->w = 13 << fbits;
	col->h = 8 << fbits;
	body->grounded = 0;
	animated->id = animation_demonbird_fly_right;
	animated->frame = 0;
	animated->timer = 0;
	sprite->z = 100;
	fric->amt = player_friction << 1 | 1;
	*face = face_right;
	player->e = e;
	player->dead = 0;
	dam->hp = 1;
	strcpy(cut->handle, "player");
}

void drop_bomb(Player* player, World* world) {
	int id, px, py, vx, vy;
	CPosition* pos;
	CSprite* spr;
	CVelocity* vel;
	id = entity_index(player->e);
	pos = &world->positions[id];
	spr = &world->sprites[id];
	vel = &world->velocities[id];
	vx = vel->x;
	vy = vel->y;
	px = pos->x + (spr->rect.w << fbits) / 2;
	py = pos->y + (spr->rect.h << fbits);
	new_bomb(world, px, py, vx, vy);
}

void update_player(Player* player, World* world) {
	CAnimated* animated;
	CFace* face;
	CPosition* pos;
	CVelocity* vel;
	CBody* body;
	int id;
	Animation_ID nanim;
	if (player->dead) { return; }
	id = entity_index(player->e);
	animated = &world->animated[id];
	face = &world->faces[id];
	pos = &world->positions[id];
	vel = &world->velocities[id];
	body = &world->bodies[id];
	if (btn_pressed(btn_jump))
		vel->y = -4 << fbits;
	if (btn_pressed(btn_left))
		vel->x -= 1 << fbits;
	if (btn_pressed(btn_right))
		vel->x += 1 << fbits;
	nanim = animated->id;
	if (body->grounded < 3) {
		if (*face == face_left) {
			nanim = animation_demonbird_idle_left;
		} else {
			nanim = animation_demonbird_idle_right;
		}
	} else {
		if (vel->x < 0) {
			*face = face_left;
			nanim = animation_demonbird_fly_left;
		} else if (vel->x > 0) {
			*face = face_right;
			nanim = animation_demonbird_fly_right;
		} else {
			nanim = animation_demonbird_fly_right;
		}
	}
	if (nanim != animated->id) {
		animated->frame = 0;
		animated->timer = 0;
		animated->id = nanim;
	}
	if (
		pos->x < 0 ||
		pos->y < 0 ||
		pos->x >> fbits > renderer_w ||
		pos->y >> fbits > renderer_h
	) {
		kill_player(world, player);
	}
	if (
		btn_pressed(btn_shoot) &&
		world->persistent.inv & (1 << item_bomb)
	) {
		drop_bomb(player, world);
	}
#ifdef DEBUG
#if enable_mouse
	if (
		btn_pressed(btn_pc_shift) &&
		mbtn_just_pressed(mbtn_left)
	) {
		pos->x = get_mouse_x();
		pos->y = get_mouse_y();
/*		pos->x /= default_window_scale;
		pos->y /= default_window_scale;*/
		pos->x <<= fbits;
		pos->y <<= fbits;
	}
#endif
#endif
}

void kill_player(World* world, Player* player) {
	int id;
	Entity e;
	CPosition* pos;
	e = player->e;
	id = entity_index(e);
	pos = &world->positions[id];
	new_blood_splatter(
		world,
		clamp(pos->x, 0, renderer_w << fbits),
		clamp(pos->y, 0, renderer_h << fbits),
		0,
		-max_gore_velo
	);
	destroy_entity(world, e);
	player->dead = 1;
}

void pickup_item(
	struct World* world,
	Entity p
) {
	int pid, m;
	CPickup* pickup;
	pid = entity_index(p);
	pickup = &world->pickups[pid];
	m = 1 << pickup->item;
	world->persistent.inv |= m;
	destroy_entity(world, p);
}
