#ifndef render_h
#define render_h

#include "config.h"
#include "rect.h"

typedef struct {
	unsigned char r, g, b, a;
} Colour;

Colour make_colour(unsigned rgb, unsigned char a);
Colour make_black();
Colour make_white();

typedef struct Bitmap {
	Colour* pixels;
	int w, h;
} Bitmap;

void init_bitmap(
	Bitmap* bitmap,
	Colour* pixels,
	int w,
	int h
);

typedef enum {
	bm_font_range_printable,
	bm_font_range_full
} BM_Font_Range;

/* Rectangles should come after the font. */
/* TODO Maybe some generic interface so that both fixed
 * width and regular fonts can be rendered through the
 * same calls? */
typedef struct BM_Font {
	const Bitmap* bmp;
	int height;
	short range, first;
} BM_Font;

int bm_font_size(BM_Font_Range range);

void get_font_rect(
	const BM_Font* font,
	Rectangle* dst,
	char c
);

void init_bm_font(
	BM_Font* font,
	Bitmap* bmp,
	BM_Font_Range range
);
Rectangle text_rect(const BM_Font* font, const char* text);

void rfont_char(const BM_Font* font, int x, int y, char c);
void rfont_char_col(
	const BM_Font* font,
	int x,
	int y,
	char c,
	Colour colour
);
void rfont_text(
	const BM_Font* font,
	int x,
	int y,
	const char* text
);
void rfont_text_col(
	const BM_Font* font,
	int x,
	int y,
	const char* text,
	Colour colour
);

typedef struct Fixed_Font {
	const Bitmap* bmp;
	int char_w, char_h;
	int space;
} Fixed_Font;

void init_fixed_font(
	Fixed_Font* font,
	const Bitmap* bmp,
	int char_w,
	int char_h,
	int space
);
void rffont_char(
	const Fixed_Font* font,
	int x,
	int y,
	char c
);
void rffont_char_col(
	const Fixed_Font* font,
	int x,
	int y,
	char c,
	Colour colour
);
void rffont_text(
	const Fixed_Font* font,
	int x,
	int y,
	const char* text
);
void rffont_text_col(
	const Fixed_Font* font,
	int x,
	int y,
	const char* text,
	Colour colour
);

Rectangle ftext_rect(const Fixed_Font* font, const char* text);

void render_init(void);

void render_begin(void);
void render_clear(void);
void render_clear_col(Colour col);
void render_clip(const Rectangle* rect);
void render_reset_clip(void);

Colour blend(Colour dst, Colour src);
Colour blend_mod(Colour dst, Colour src, Colour mod);

Colour* get_render_pixels(void);

extern int sprite_camera_x;
extern int sprite_camera_y;

void render_bitmap(
	const Bitmap* bitmap,
	int x,
	int y,
	const Rectangle* rect
);

void render_bitmap_col(
	const Bitmap* bitmap,
	int x,
	int y,
	const Rectangle* rect,
	Colour colour
);

void render_rect(
	const Rectangle* rect,
	Colour colour
);

void render_mask(
	const unsigned char* pixels,
	int x,
	int y,
	int w,
	int h,
	const Rectangle* rect,
	Colour colour
);

void rcopy(
	Bitmap* dst,
	const Bitmap* src,
	int x,
	int y,
	const Rectangle* rect
);

void rcopy_col(
	Bitmap* dst,
	const Bitmap* src,
	int x,
	int y,
	const Rectangle* rect,
	Colour colour
);

void rcopy_ac(
	Bitmap* dst,
	const Bitmap* src,
	int x,
	int y,
	const Rectangle* rect,
	unsigned char t
);

void rcopy_rect(
	Bitmap* dst,
	const Rectangle* rect,
	Colour colour
);

#endif
