#include <string.h>

#include "components.h"
#include "sprite.h"

static const Sprite sprites[] = {
	/* sprite_blood_small */
	{
		asset_id_fx,
		{ 0, 2, 1, 1 }
	},
	/* sprite_blood_big */
	{
		asset_id_fx,
		{ 0, 0, 2, 2 }
	},
	/* sprite_organ_a */
	{
		asset_id_fx,
		{ 2, 0, 5, 4 }
	},
	/* sprite_organ_b */
	{
		asset_id_fx,
		{ 7, 0, 2, 3 }
	},
	/* sprite_organ_c */
	{
		asset_id_fx,
		{ 9, 0, 2, 3 }
	},
	/* sprite_icon_close */
	{
		asset_id_ui,
		{ 0, 0, 4, 4 }
	},
	/* sprite_icon_check */
	{
		asset_id_ui,
		{ 4, 0, 4, 4 }
	},
	/* sprite_icon_expanded */
	{
		asset_id_ui,
		{ 8, 0, 4, 4 }
	},
	/* sprite_icon_collapsed */
	{
		asset_id_ui,
		{ 12, 0, 4, 4 }
	},
	/* sprite_tile_box */
	{
		asset_id_map,
		{ 0, 0, 16, 16 }
	},
	/* sprite_copcar_left */
	{
		asset_id_prop,
		{ 0, 19, 44, 14 }
	},
	/* sprite_copcar_right */
	{
		asset_id_prop,
		{ 44, 19, 44, 14 }
	},
	/* sprite_cop_left */
	{
		asset_id_prop,
		{ 13, 0, 13, 15 }
	},
	/* sprite_cop_right */
	{
		asset_id_prop,
		{ 0, 0, 13, 15 }
	},
	/* sprite_cop_bullet */
	{
		asset_id_arms,
		{ 0, 0, 2, 2 }
	},
	/* sprite_spike_a */
	{
		asset_id_prop,
		{ 48, 0, 16, 16 }
	},
	/* sprite_spike_b */
	{
		asset_id_prop,
		{ 64, 0, 16, 16 }
	},
	/* sprite_spike_c */
	{
		asset_id_prop,
		{ 0, 48, 16, 16 }
	},
	/* sprite_star_blue_a */
	{
		asset_id_prop,
		{ 33, 0, 5, 5 }
	},
	/* sprite_star_blue_b */
	{
		asset_id_prop,
		{ 38, 3, 3, 3 }
	},
	/* sprite_star_blue_c */
	{
		asset_id_prop,
		{ 41, 1, 1, 1 }
	},
	/* sprite_star_yellow_a */
	{
		asset_id_prop,
		{ 33, 5, 5, 5 }
	},
	/* sprite_star_yellow_b */
	{
		asset_id_prop,
		{ 38, 0, 3, 3 }
	},
	/* sprite_star_yellow_c */
	{
		asset_id_prop,
		{ 41, 0, 1, 1 }
	},
	/* sprite_game_over */
	{
		asset_id_ui,
		{ 0, 20, 180, 29 }
	},
	/* sprite_bomb */
	{
		asset_id_arms,
		{ 2, 0, 9, 5 }
	},
	/* sprite_bomb_down */
	{
		asset_id_arms,
		{ 11, 0, 5, 9 }
	},
	/* sprite_floppy */
	{
		asset_id_prop,
		{ 16, 48, 16, 16 }
	},
	/* sprite_drugs_sign */
	{
		asset_id_prop,
		{ 32, 36, 22, 28 }
	},
	/* sprite_dealer */
	{
		asset_id_prop,
		{ 68, 49, 9, 15 }
	},
	/* sprite_face_null */
	{
		asset_id_face,
		{ 64, 0, 64, 64 }
	},
	/* sprite_face_border */
	{
		asset_id_face,
		{ 0, 0, 64, 64 }
	},
	/* sprite_face_player */
	{
		asset_id_face,
		{ 128, 0, 64, 64 }
	},
	/* sprite_face_dealer */
	{
		asset_id_face,
		{ 192, 0, 64, 64 }
	},
	/* sprite_face_cia */
	{
		asset_id_face,
		{ 256, 0, 64, 64 }
	},
	/* sprite_cia_head */
	{
		asset_id_boss,
		{ 0, 0, 55, 64 }
	},
	/* sprite_cia_head_hurt */
	{
		asset_id_boss,
		{ 55, 0, 55, 64 }
	},
	/* sprite_cia_head_fire */
	{
		asset_id_boss,
		{ 110, 0, 55, 64 }
	},
	/* sprite_cia_body */
	{
		asset_id_boss,
		{ 0, 64, 114, 109 }
	},
	/* sprite_border_blood */
	{
		asset_id_ui,
		{ 180, 29, 16, 16 }
	},
	/* sprite_border_cross */
	{
		asset_id_ui,
		{ 196, 29, 16, 16 }
	},
	/* sprite_boss_hb_left */
	{
		asset_id_ui,
		{ 180, 20, 9, 9 }
	},
	/* sprite_boss_hb_middle */
	{
		asset_id_ui,
		{ 189, 20, 1, 9 }
	},
	/* sprite_boss_hb_middle_full */
	{
		asset_id_ui,
		{ 190, 20, 1, 9 }
	},
	/* sprite_boss_hb_right */
	{
		asset_id_ui,
		{ 191, 20, 29, 9 }
	},
	/* sprite_cia_bullet */
	{
		asset_id_arms,
		{ 16, 0, 4, 4 }
	},
	/* sprite_cloud */
	{
		asset_id_ui,
		{ 0, 51, 135, 26 }
	},
	/* sprite_cracked_wall */
	{
		asset_id_prop,
		{ 91, 0, 18, 96 }
	}
};

static const char* sprite_names[sprite_count] = {
#define x(name) #name,
	sprites_xmacro()
#undef x
};

void init_csprite(CSprite* sprite, Sprite_ID id) {
	const Sprite* s;

	s = &sprites[id];
	sprite->asset = s->bitmap;
	sprite->rect  = s->rect;
	sprite->z = 0;
}

const Sprite* get_sprite(Sprite_ID id) {
	return &sprites[id];
}

const char* get_sprite_name(Sprite_ID id) {
	return sprite_names[id];
}

Sprite_ID get_sprite_id(const char* name) {
	int i;
	for (i = 0; i < sprite_count; i++) {
		if (!strcmp(name, sprite_names[i])) {
			return (Sprite_ID)i;
		}
	}
	return sprite_count;
}
