#include <stdlib.h>

#include "asset.h"
#include "components.h"
#include "maths.h"
#include "profile.h"
#include "rcache.h"
#include "render.h"
#include "sprite_system.h"
#include "world.h"

int sprite_compare(const void* av, const void* bv) {
	const Sprite_RD* a, * b;
	a = av;
	b = bv;
	return a->z - b->z;
}

void sprite_system(Sprite_System* sp, const World* world) {
	int i, c;
	Com_Bitset bits, mask;
	const CSprite* sprite;
	const CPosition* pos;
	Sprite_RD* rd;
	profiler_begin("sprite_system");
	sp->count = 0;
	general_system(ctype_position | ctype_sprite)
		pos = &world->positions[i];
		sprite = &world->sprites[i];
		rd = &sp->sorted[sp->count++];
		rd->x = pos->x >> fbits;
		rd->y = pos->y >> fbits;
		rd->z = sprite->z;
		rd->rect = sprite->rect;
		rd->bitmap = get_bitmap(sprite->asset);
	}

	qsort(
		sp->sorted,
		sp->count,
		sizeof *sp->sorted,
		sprite_compare
	);

	c = sp->count;
	for (i = 0; i < c; i++) {
		rd = &sp->sorted[i];
		rc_add_cmd_bmp(
			rd->bitmap,
			rd->x,
			rd->y,
			&rd->rect
		);
	}
	profiler_end();
}

void animation_system(World* world) {
	int i;
	Com_Bitset bits, mask;
	CSprite* sprite;
	CAnimated* animated;
	const Animation* animation;

	general_system(ctype_sprite | ctype_animated)
		sprite = &world->sprites[i];
		animated = &world->animated[i];
		animation = get_animation(animated->id);

		sprite->asset = animation->bitmap;

		animated->timer++;
		if (animated->timer > animation->slowness) {
			animated->frame++;
			animated->timer = 0;
		}

		if (animated->frame >= animation->frame_count) {
			if (world->tags[i] & tag_destroy_on_anim_done) {
				destroy_entity(world, world->entities[i]);
				continue;
			}

			animated->frame = 0;
			animated->timer = 0;
		}

		sprite->rect = animation->frames[animated->frame];
	}
}
