#include "gamestate.h"
#include "levelloader.h"
#include "rcache.h"
#include "sprite_system.h"
#include "world.h"

void gamestate_transition_init(
	Gamestate* gamestate,
	World* world
) {
	(void)world;
	gamestate->frame = 0;
	gamestate->tanim = animation_transition_fade_out;
}

void gamestate_transition_update(
	Gamestate* gamestate,
	World* world
) {	
	const Animation* anim;
	anim = get_animation(gamestate->tanim);
	if (gamestate->frame >= anim->frame_count) {
		if (gamestate->tanim == animation_transition_fade_out) {
			gamestate->tanim = animation_transition_fade_in;
			if (gamestate->trans[0] == ' ')
				load_game(world, savefile_name);
			else if (gamestate->trans[0])
				load_level(gamestate->trans, world);
			init_gamestate_t(gamestate, world, gamestate->tto);
			gamestate->frame = 0;
		} else {
			switch_gamestate(
				gamestate,
				world,
				gamestate->tto
			);
			gamestate->frame = anim->frame_count - 1;
		}
	}
}

void gamestate_transition_render(
	Gamestate* gamestate,
	World* world
) {
	const Animation* anim;
	const Rectangle* rect;
	int x, y, w, h;
	(void)gamestate;
	if (gamestate->tanim == animation_transition_fade_out) {
		render_gamestate_t(gamestate, world, gamestate->prev);
	} else {
		render_gamestate_t(gamestate, world, gamestate->tto);
	}

	anim = get_animation(gamestate->tanim);
	rect = &anim->frames[gamestate->frame++];

	w = renderer_w / rect->w;
	h = renderer_h / rect->h;
	for (y = 0; y < h; y++)
		for (x = 0; x < w; x++)
			rc_add_cmd_bmp(
				get_bitmap(anim->bitmap),
				x * rect->w,
				y * rect->h,
				rect
			);
}

void transition_level(
	Gamestate* gamestate,
	const char* name,
	struct World* world,
	Gamestate_Type gs
) {
	const char* s;
	char* d;
	for (
		s = name, d = gamestate->trans;
		*s;
		s++, d++
	) *d = *s;
	*d = 0;
	gamestate->tto = gs;
	switch_gamestate(
		gamestate,
		world,
		gamestate_type_transition
	);
}

void transition_load(
	Gamestate* gamestate,
	World* world
) {
	gamestate->trans[0] = ' ';
	gamestate->tto = gamestate_type_general;
	switch_gamestate(
		gamestate,
		world,
		gamestate_type_transition
	);
}
