#include "config.h"
#include "plat.h"
#include "world.h"

void init_world(World* world, unsigned seed) {
	char* b;
	int i, s;
#if defined(DEBUG) && randomise_world
	b = (char*)world;
	s = sizeof *world;
	for (i = 0; i < s; i++)
		b[i] = (char)rng();
#endif
	world->seed = seed;
	world->avail = -1;
	world->created_count = 0;
	world->entity_count = 0;
	b = (char*)&world->persistent;
	s = sizeof world->persistent;
	for (i = 0; i < s; i++)
		b[i] = 0;
#if force_bomb
	world->persistent.inv |= 1 << item_bomb;
#endif
}

Entity new_entity(World* world) {
	int id, v, cid, cv;
	Entity e;
	world->entity_count++;
	if (world->entity_count > max_entities) {
		print_err("Reached entity limit.\n");
		pbreak(error_out_of_memory);
	}
	if (world->avail > 0) {
		cid = world->avail;
		e = world->entities[cid];
		cv = entity_version(e);
		world->avail = entity_index(e);
		e = make_entity(cid, cv);
		world->entities[cid] = e;
		return e;
	}
	id = world->created_count++;
	world->tags[id]   = 0;
	world->bitset[id] = 0;
	v = 0;
	e = make_entity(id, v);
	world->entities[id] = e;
	return e;
}

void destroy_entity(World* world, Entity e) {
	int id, v;
	CList* list;
	id = entity_index(e);
	if (world->bitset[id] & ctype_list) {
		list = &world->lists[id];
		destroy_entity(world, list->next);
	}
	v = entity_version(e);
	v++;
	world->tags[id]   = 0;
	world->bitset[id] = 0;
	world->entity_count--;
	world->entities[id] = make_entity(world->avail, v);
	world->avail = id;
}

void add_components(World* world, Entity e, Com_Bitset bits) {
	int id;
	id = entity_index(e);
	world->bitset[id] |= bits;
}

void remove_components(World* world, Entity e, Com_Bitset bits) {
	int id;
	id = entity_index(e);
	world->bitset[id] &= ~bits;
}

void add_tags(World* world, Entity e, Tags tags) {
	int id;
	id = entity_index(e);
	world->tags[id] |= tags;
}

void remove_tags(World* world, Entity e, Tags tags) {
	int id;
	id = entity_index(e);
	world->tags[id] &= ~tags;
}

#if enable_savegame
int load_game(
	World* world,
	const char* name
) {
	File* file;
	file = file_open(
		name,
		file_flags_read
	);
	if (!file) return 0;
	file_read(file, world, sizeof* world);
	file_close(file);
	return 1;
}

int save_game(
	const World* world,
	const char* name
) {
	File* file;
	file = file_open(
		name,
		file_flags_write | file_flags_create
	);
	if (!file) return 0;
	file_write(file, world, sizeof* world);
	file_close(file);
	return 1;
}
#endif
