#ifndef world_h
#define world_h

#include "components.h"
#include "config.h"
#include "entity.h"
#include "map.h"
#include "player.h"
#include "tags.h"

#define general_system_vars \
	Com_Bitset mask, bits; \
	int i;
#define general_system(m_) \
	mask = m_; \
	for (i = 0; i < world->created_count; i++) { \
		bits = world->bitset[i]; \
		if ((bits & mask) != mask) { continue; } \

#define Com_Pool(t_, n_) t_ n_[max_entities]

typedef unsigned Com_Bitset;

typedef struct {
	int inv;
} World_Persistent;

typedef struct World {
	Com_Pool(CAnimated, animated);
	Com_Pool(CBody,     bodies);
	Com_Pool(CCollider, colliders);
	Com_Pool(CFace,     faces);
	Com_Pool(CFollow,   follows);
	Com_Pool(CLifetime, lifetimes);
	Com_Pool(CPosition, positions);
	Com_Pool(CSprite,   sprites);
	Com_Pool(CVelocity, velocities);
	Com_Pool(CDamageable, damageables);
	Com_Pool(CFriction, frictions);
	Com_Pool(CMonster,  monsters);
	Com_Pool(CCorpse,   corpses);
	Com_Pool(CList,     lists);
	Com_Pool(CConttrig, conttrigs);
	Com_Pool(CPickup,   pickups);
	Com_Pool(CCutscene, cutscenes);
	Com_Pool(CCuttrig,  cuttrigs);
	Com_Pool(CBoss,     bosses);
	Com_Pool(CImpact,   impacts);

	Com_Bitset bitset[max_entities];
	Tags tags[max_entities];
	Entity entities[max_entities];

	Map map;
	Player player;

	Entity avail;
	int created_count;
	int entity_count;
	unsigned seed;

	World_Persistent persistent;
} World;

void init_world(World* world, unsigned seed);

Entity new_entity(World* world);
void destroy_entity(World* world, Entity e);
void add_components(World* world, Entity e, Com_Bitset bits);
void remove_components(World* world, Entity e, Com_Bitset bits);
void add_tags(World* world, Entity e, Tags tags);
void remove_tags(World* world, Entity e, Tags tags);

#if enable_savegame
int load_game(
	World* world,
	const char* name
);
int save_game(
	const World* world,
	const char* name
);
#endif

#endif
